% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Incremental.R
\name{cleanNonIncremental}
\alias{cleanNonIncremental}
\title{Removes csv files from the execution folder as there should be no csv files
when running in non-incremental model}
\usage{
cleanNonIncremental(executionFolder)
}
\arguments{
\item{executionFolder}{The folder that has the execution files}
}
\value{
A list with the settings
}
\description{
Removes csv files from the execution folder as there should be no csv files
when running in non-incremental model
}
\examples{
# example code

cleanNonIncremental(file.path(tempdir(), 'incremental'))

}
\seealso{
Other Incremental: 
\code{\link{cleanIncremental}()}
}
\concept{Incremental}
