% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TargetAnalysis.R
\name{createTargetBaselineSettings}
\alias{createTargetBaselineSettings}
\title{Create target baseline aggregate covariate study settings}
\usage{
createTargetBaselineSettings(
  targetIds,
  limitToFirstInNDays = 99999,
  minPriorObservation = 0,
  covariateSettings = FeatureExtraction::createCovariateSettings(useDemographicsGender =
    TRUE, useDemographicsAge = TRUE, useDemographicsAgeGroup = TRUE, useDemographicsRace
    = TRUE, useDemographicsEthnicity = TRUE, useDemographicsIndexYear = TRUE,
    useDemographicsIndexMonth = TRUE, useDemographicsTimeInCohort = TRUE,
    useDemographicsPriorObservationTime = TRUE, useDemographicsPostObservationTime =
    TRUE, useConditionGroupEraLongTerm = TRUE, useDrugGroupEraOverlapping = TRUE,
    useDrugGroupEraLongTerm = TRUE, useProcedureOccurrenceLongTerm = TRUE, 
    
    useMeasurementLongTerm = TRUE, useObservationLongTerm = TRUE,
    useDeviceExposureLongTerm = TRUE, useVisitConceptCountLongTerm = TRUE,
    useConditionGroupEraShortTerm = TRUE, useDrugGroupEraShortTerm = TRUE,
    useProcedureOccurrenceShortTerm = TRUE, useMeasurementShortTerm = TRUE,
    useObservationShortTerm = TRUE, useDeviceExposureShortTerm = TRUE,
    useVisitConceptCountShortTerm = TRUE, useCharlsonIndex = TRUE, endDays = 0,
    longTermStartDays = -365, shortTermStartDays = -30)
)
}
\arguments{
\item{targetIds}{A list of cohortIds for the target cohorts}

\item{limitToFirstInNDays}{Whether to remove target cohort entries that occur within limitToFirstInNDays of a prior entry. limitToFirstInNDays = 99999 means limit to first entry.}

\item{minPriorObservation}{The minimum time (in days) in the database a patient in the target cohorts must be observed prior to index}

\item{covariateSettings}{An object created using \code{FeatureExtraction::createCovariateSettings}}
}
\value{
A list with the settings
}
\description{
Create target baseline aggregate covariate study settings
}
\examples{

aggregateSetting <- createTargetBaselineSettings(
  targetIds = c(1,2),
  limitToFirstInNDays = 99999,
  minPriorObservation = 365
)

}
\seealso{
Other Aggregate: 
\code{\link{createCaseSeriesSettings}()},
\code{\link{createRiskFactorSettings}()}
}
\concept{Aggregate}
