% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TimeToEvent.R
\name{createTimeToEventSettings}
\alias{createTimeToEventSettings}
\title{Create time to event study settings}
\usage{
createTimeToEventSettings(targetIds, outcomeIds)
}
\arguments{
\item{targetIds}{A list of cohortIds for the target cohorts}

\item{outcomeIds}{A list of cohortIds for the outcome cohorts}
}
\value{
An list with the time to event settings
}
\description{
Create time to event study settings
}
\examples{
# example code

tteSet <- createTimeToEventSettings(
  targetIds = c(1,2),
  outcomeIds = 3
)


}
\seealso{
Other TimeToEvent: 
\code{\link{computeTimeToEventAnalyses}()}
}
\concept{TimeToEvent}
