% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Database.R
\name{createSqliteDatabase}
\alias{createSqliteDatabase}
\title{Create an sqlite database connection}
\usage{
createSqliteDatabase(sqliteLocation = tempdir())
}
\arguments{
\item{sqliteLocation}{The location of the sqlite database}
}
\value{
Returns the connection detail object to the sqlite database
}
\description{
This function creates a connection to an sqlite database
}
\details{
This function creates a sqlite database and connection
}
\examples{

charResultDbCD <- createSqliteDatabase()


}
\seealso{
Other Database: 
\code{\link{createCharacterizationTables}()},
\code{\link{insertResultsToDatabase}()}
}
\concept{Database}
