% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Incremental.R
\name{cleanIncremental}
\alias{cleanIncremental}
\title{Removes csv files from folders that have not been marked as completed
and removes the record of the execution file}
\usage{
cleanIncremental(executionFolder, ignoreWhenEmpty = FALSE)
}
\arguments{
\item{executionFolder}{The folder that has the execution files}

\item{ignoreWhenEmpty}{When TRUE, if there are no incremental logs then nothing is run}
}
\value{
A list with the settings
}
\description{
Removes csv files from folders that have not been marked as completed
and removes the record of the execution file
}
\examples{

cleanIncremental(
  file.path(tempdir(), 'incremental'),
  ignoreWhenEmpty = TRUE
)


}
\seealso{
Other Incremental: 
\code{\link{cleanNonIncremental}()}
}
\concept{Incremental}
