% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Characterization.R
\docType{package}
\name{Characterization-package}
\alias{Characterization}
\alias{Characterization-package}
\title{Characterization: Implement Descriptive Studies Using the Common Data Model}
\description{
An end-to-end framework that enables users to implement various descriptive studies for a given set of target and outcome cohorts for data mapped to the Observational Medical Outcomes Partnership Common Data Model.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://ohdsi.github.io/Characterization/}
  \item \url{https://github.com/OHDSI/Characterization}
  \item Report bugs at \url{https://github.com/OHDSI/Characterization/issues}
}

}
\author{
\strong{Maintainer}: Jenna Reps \email{jreps@its.jnj.com}

Authors:
\itemize{
  \item Patrick Ryan \email{ryan@ohdsi.org}
  \item Chris Knoll \email{knoll@ohdsi.org}
}

}
\keyword{internal}
