% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DechallengeRechallenge.R
\name{createDechallengeRechallengeSettings}
\alias{createDechallengeRechallengeSettings}
\title{Create dechallenge rechallenge study settings}
\usage{
createDechallengeRechallengeSettings(
  targetIds,
  outcomeIds,
  dechallengeStopInterval = 30,
  dechallengeEvaluationWindow = 30
)
}
\arguments{
\item{targetIds}{A list of cohortIds for the target cohorts}

\item{outcomeIds}{A list of cohortIds for the outcome cohorts}

\item{dechallengeStopInterval}{An integer specifying the how much time to add to the cohort_end when determining whether the event starts during cohort and ends after}

\item{dechallengeEvaluationWindow}{An integer specifying the period of time after the cohort_end when you cannot see an outcome for a dechallenge success}
}
\value{
A list with the settings
}
\description{
Create dechallenge rechallenge study settings
}
\examples{
drSet <- createDechallengeRechallengeSettings(
  targetIds = c(1,2),
  outcomeIds = 3
)


}
\seealso{
Other DechallengeRechallenge: 
\code{\link{computeDechallengeRechallengeAnalyses}()},
\code{\link{computeRechallengeFailCaseSeriesAnalyses}()}
}
\concept{DechallengeRechallenge}
